package com.miya.huihua.microservicebase.mq;

import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.Producer;
import com.aliyun.openservices.ons.api.SendCallback;
import com.aliyun.openservices.ons.api.SendResult;
import com.aliyun.openservices.shade.com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

import static com.miya.huihua.microservicebase.common.Constants.TAG_ALL;

/**
 * @author zhouxu
 * @createTime 2020/01/17
 */

@Service
@Slf4j
public class AliProducerImpl implements AliProducer {

    @Autowired(required = false)
    private OnsManager onsManager;


    @Override
    public void putEvent(String topic, Object o, String... tags) {

        try {
            SendResult sendResult = sendMessage(topic, getMessage(topic, o, tags));
            if (sendResult != null) {
                log.info("send msg success,Topic is :{},getMessageId is :{}", sendResult.getTopic(), sendResult.getMessageId());
            }
        } catch (Exception e) {
            log.error("Send mq message failed. Topic is:{}, Reson is : {}", topic, e.getMessage());
            e.printStackTrace();
        }
    }

    @Override
    public void putAsyncEvent(String topic, Object o, SendCallback sendCallback, String... tags) {
        sendAsyncMessage(topic, getMessage(topic, o, tags), sendCallback);
    }

    @Override
    public void putDelayEvent(String topic, Object o, SendCallback sendCallback, Long delayTime, String... tags) {
        Message message = getMessage(topic, o, tags);
        message.setStartDeliverTime(delayTime);
        if (sendCallback != null) {
            sendAsyncMessage(topic, getMessage(topic, o, tags), sendCallback);
        } else {
            SendResult sendResult = sendMessage(topic, getMessage(topic, o, tags));
            if (sendResult != null) {
                log.info("send msg success,Topic is :{},getMessageId is :{}", sendResult.getTopic(), sendResult.getMessageId());
            }
        }
    }

    /**
     * 发送异步消息
     *
     * @param topic
     * @param message
     * @param sendCallback
     */
    public void sendAsyncMessage(String topic, Message message, SendCallback sendCallback) {
        Producer producer = getProducer(topic);
        producer.sendAsync(message, sendCallback);
    }

    /**
     * 发送同步消息
     *
     * @param topic
     * @param message
     * @return
     */
    public SendResult sendMessage(String topic, Message message) {
        Producer producer = getProducer(topic);
        SendResult sendResult = producer.send(message);
        return sendResult;
    }


    private Message getMessage(String topic, Object o, String... tags) {
        Message message = new Message(topic, tags.length < 1 ? TAG_ALL : Arrays.asList(tags).stream().collect(Collectors.joining("||")), JSON.toJSONString(o).getBytes());
        return message;
    }

    private Producer getProducer(String topic) {
        ConcurrentHashMap<String, Producer> concurrentHashMap = onsManager.getProducerConcurrentHashMap();
        return concurrentHashMap.get(topic);
    }
}
